@@ -22,12 +22,16 @@ def run_subtool(parser, args):
         import fastq as submodule
     elif args.command == 'hist':
         import hist as submodule
+    elif args.command == 'metadata':
+        import metadata as submodule
     elif args.command == 'nucdist':
         import nucdist as submodule
     elif args.command == 'occupancy':
         import occupancy as submodule
     elif args.command == 'qualdist':
         import qualdist as submodule
+    elif args.command == 'qualpos':
+        import qual_v_pos as submodule
     elif args.command == 'readstats':
         import readstats as submodule
     elif args.command == 'stats':
@@ -42,6 +46,10 @@ def run_subtool(parser, args):
         import winner as submodule
     elif args.command == 'yield_plot':
         import yield_plot as submodule
+    elif args.command == 'index':
+        import index as submodule
+    elif args.command == 'organise':
+        import organise as submodule
 
     # run the chosen submodule.
     submodule.run(parser, args)
@@ -94,9 +102,9 @@ def main():
     parser_fastq.add_argument('--type',
                               dest='type',
                               metavar='STRING',
-                              choices=['all', 'fwd', 'rev', '2D', 'fwd,rev'],
+                              choices=['all', 'fwd', 'rev', '2D', 'fwd,rev', 'best'],
                               default='all',
-                              help='Which type of FASTA entries should be reported? Def.=all')
+                              help='Which type of FASTQ entries should be reported? Def.=all')
     parser_fastq.add_argument('--start',
                               dest='start_time',
                               default=None,
@@ -111,12 +119,27 @@ def main():
                               dest='min_length',
                               default=0,
                               type=int,
-                              help=('Minimum read length for FASTA entry to be reported.'))
+                              help=('Minimum read length for FASTQ entry to be reported.'))
+    parser_fastq.add_argument('--max-length',
+                              dest='max_length',
+                              default=-1,
+                              type=int,
+                              help=('Maximum read length for FASTQ entry to be reported.'))                          
     parser_fastq.add_argument('--high-quality',
                               dest='high_quality',
                               default=False,
                               action='store_true',
-                              help=('Only report reads with more complement events than template.'))
+                              help=('Only report reads with more complement events than template.'))   
+    parser_fastq.add_argument('--normal-quality',
+                              dest='normal_quality',
+                              default=False,
+                              action='store_true',
+                              help=('Only report reads with fewer complement events than template.'))
+    parser_fastq.add_argument('--group',
+                              dest='group',
+                              default=0,
+                              type=int,
+                              help=('Base calling group serial number to extract, default 000'))
     parser_fastq.set_defaults(func=run_subtool)
 
 
@@ -130,7 +153,7 @@ def main():
     parser_fasta.add_argument('--type',
                               dest='type',
                               metavar='STRING',
-                              choices=['all', 'fwd', 'rev', '2D', 'fwd,rev'],
+                              choices=['all', 'fwd', 'rev', '2D', 'fwd,rev', 'best'],
                               default='all',
                               help='Which type of FASTQ entries should be reported? Def.=all')
     parser_fasta.add_argument('--start',
@@ -148,6 +171,11 @@ def main():
                               default=0,
                               type=int,
                               help=('Minimum read length for FASTA entry to be reported.'))
+    parser_fasta.add_argument('--max-length',
+                              dest='max_length',
+                              default=-1,
+                              type=int,
+                              help=('Maximum read length for FASTA entry to be reported.'))                          
     parser_fasta.add_argument('--high-quality',
                               dest='high_quality',
                               default=False,
@@ -158,6 +186,11 @@ def main():
                               default=False,
                               action='store_true',
                               help=('Only report reads with fewer complement events than template.'))
+    parser_fasta.add_argument('--group',
+                              dest='group',
+                              default=0,
+                              type=int,
+                              help=('Base calling group serial number to extract, default 000'))
     parser_fasta.set_defaults(func=run_subtool)
 
 
@@ -171,7 +204,7 @@ def main():
     parser_stats.add_argument('--type',
                               dest='type',
                               metavar='STRING',
-                              choices=['all', 'fwd', 'rev', '2D', 'fwd,rev'],
+                              choices=['all', 'fwd', 'rev', '2D', 'fwd,rev', 'best'],
                               default='all',
                               help='Which type of FASTQ entries should be reported? Def.=all')
     parser_stats.add_argument('--full-tsv',
@@ -213,8 +246,12 @@ def main():
                              dest='theme_bw',
                              default=False,
                              action='store_true',
-                             help="Use the ggplot2 black and white theme.")
-
+                             help="Use a black and white theme.")
+    parser_hist.add_argument('--watch',
+                             dest='watch',
+                             default=False,
+                             action='store_true',
+                             help="Monitor a directory.")
     parser_hist.set_defaults(func=run_subtool)
 
 
@@ -225,9 +262,14 @@ def main():
                                         help='Extract each nanopore event for each read.')
     parser_events.add_argument('files', metavar='FILES', nargs='+',
                              help='The input FAST5 files.')
+    parser_events.add_argument('--pre-basecalled',
+                              dest='pre_basecalled',
+                              default=False,
+                              action='store_true',
+                              help=('Report pre-basecalled events'))     
     parser_events.set_defaults(func=run_subtool)
 
-    
+
     ###########
     # readstats
     ###########
@@ -248,12 +290,11 @@ def main():
     parser_tabular.add_argument('--type',
                               dest='type',
                               metavar='STRING',
-                              choices=['all', 'fwd', 'rev', '2D', 'fwd,rev'],
+                              choices=['all', 'fwd', 'rev', '2D', 'fwd,rev', 'best'],
                               default='all',
                               help='Which type of FASTA entries should be reported? Def.=all')
     parser_tabular.set_defaults(func=run_subtool)
 
-    
     #########
     # nucdist
     #########
@@ -263,6 +304,29 @@ def main():
                              help='The input FAST5 files.')
     parser_nucdist.set_defaults(func=run_subtool)
 
+    #########
+    # metadata
+    #########
+    parser_metadata = subparsers.add_parser('metadata',
+                                        help='Return run metadata such as ASIC ID and temperature from a set of FAST5 files')
+    parser_metadata.add_argument('files', metavar='FILES', nargs='+',
+                             help='The input FAST5 files.')
+    parser_metadata.add_argument('--read',
+                              dest='read',
+                              default=False,
+                              action='store_true',
+                              help=('Report read level metadata'))      
+    parser_metadata.set_defaults(func=run_subtool)
+    
+    #########
+    # index
+    #########
+    parser_index = subparsers.add_parser('index',
+                                        help='Tabulate all file location info and metadata such as ASIC ID and temperature from a set of FAST5 files')
+    parser_index.add_argument('files', metavar='FILES', nargs='+',
+                             help='The input FAST5 files.')
+    parser_index.set_defaults(func=run_subtool)
+
     
     ##########
     # qualdist
@@ -274,6 +338,61 @@ def main():
     parser_qualdist.set_defaults(func=run_subtool)
 
 
+
+    ##########
+    # qual vs. position
+    ##########
+    parser_qualpos = subparsers.add_parser('qualpos',
+                                        help='Get the qual score distribution over positions in reads')
+    parser_qualpos.add_argument('files', metavar='FILES', nargs='+',
+                             help='The input FAST5 files.')
+    parser_qualpos.set_defaults(func=run_subtool)
+    parser_qualpos.add_argument('--min-length',
+                              dest='min_length',
+                              default=0,
+                              type=int,
+                              help=('Minimum read length to be included in analysis.'))
+    parser_qualpos.add_argument('--max-length',
+                              dest='max_length',
+                              default=1000000000,
+                              type=int,
+                              help=('Maximum read length to be included in analysis.'))
+    parser_qualpos.add_argument('--bin-width',
+                              dest='bin_width',
+                              default=1000,
+                              type=int,
+                              help=('The width of bins (default: 1000 bp).'))
+    parser_qualpos.add_argument('--type',
+                              dest='type',
+                              metavar='STRING',
+                              choices=['all', 'fwd', 'rev', '2D', 'fwd,rev', 'best'],
+                              default='all',
+                              help='Which type of reads should be analyzed? Def.=all, choices=[all, fwd, rev, 2D, fwd,rev, best]')
+    parser_qualpos.add_argument('--start',
+                              dest='start_time',
+                              default=None,
+                              type=int,
+                              help='Only analyze reads from after start timestamp')
+    parser_qualpos.add_argument('--end',
+                              dest='end_time',
+                              default=None,
+                              type=int,
+                              help='Only analyze reads from before end timestamp')
+    parser_qualpos.add_argument('--high-quality',
+                              dest='high_quality',
+                              default=False,
+                              action='store_true',
+                              help='Only analyze reads with more complement events than template.')
+
+    parser_qualpos.add_argument('--saveas',
+                             dest='saveas',
+                             metavar='STRING',
+                             help='''Save the plot to a file named filename.extension (e.g. pdf, jpg)''',
+                             default=None)
+
+
+
+
     ##########
     # winner
     ##########
@@ -284,7 +403,7 @@ def main():
     parser_winner.add_argument('--type',
                               dest='type',
                               metavar='STRING',
-                              choices=['all', 'fwd', 'rev', '2D', 'fwd,rev'],
+                              choices=['all', 'fwd', 'rev', '2D', 'fwd,rev', 'best'],
                               default='all',
                               help='Which type of FASTA entries should be reported? Def.=all')
     parser_winner.set_defaults(func=run_subtool)
@@ -312,7 +431,7 @@ def main():
                              dest='theme_bw',
                              default=False,
                              action='store_true',
-                             help="Use the ggplot2 black and white theme.")
+                             help="Use a black and white theme.")
 
     parser_squiggle.set_defaults(func=run_subtool)
 
@@ -347,12 +466,7 @@ def main():
                              dest='theme_bw',
                              default=False,
                              action='store_true',
-                             help="Use the ggplot2 black and white theme.")
-    parser_yield_plot.add_argument('--extrapolate',
-                             dest='extrapolate',
-                             metavar='INTEGER',
-                             default=0,
-                             help="Fit a curve and extrapolate to n hours")
+                             help="Use a black and white theme.")
     parser_yield_plot.add_argument('--skip',
                              dest='skip',
                              metavar='INTEGER',
@@ -364,7 +478,7 @@ def main():
                              metavar='STRING',
                              help='Save the data frame used to construct plot to a file.',
                              default=None)
-                             
+
     parser_yield_plot.set_defaults(func=run_subtool)
 
     ############
@@ -386,8 +500,26 @@ def main():
                              help='The type of plot to generate',
                              default='read_count')
 
-
     parser_occupancy.set_defaults(func=run_subtool)
+    
+    ##########
+    # organise
+    ##########
+
+    parser_organise = subparsers.add_parser('organise',
+                               help='Move FAST5 files into a useful folder hierarchy')
+    parser_organise.add_argument('files', metavar='FILES', nargs='+',
+                               help='The input FAST5 files.')
+    parser_organise.add_argument('dest',
+                               metavar='STRING',
+                               help='The destination directory.')
+    parser_organise.add_argument('--copy',
+                               default=False,
+                               action='store_true',
+                               dest='copy', 
+                               help='Make a copy of files instead of moving')
+
+    parser_organise.set_defaults(func=run_subtool)
 
     #######################################################
     # parse the args and call the selected function
